from asaaspy.schemas.v3.subaccount import SubAccountSchema
from schemas.v3.subaccount import SubAccountViewSchema


class TestSubAccountResource:
    def test_create_minimal_subaccount_physical_person(self, asaas_svc):
        request_subaccount = SubAccountSchema(
            name="Porco Rosso",
            email="porco.rosso@asaaspy.com.br",
            cpfCnpj="751.493.120-10",
            mobilePhone="41991524243",
            incomeValue=10000,
            address="Avenida Pres. Epitácio Pessoa",
            addressNumber="2690",
            province="Tambauzinho",
            postalCode="58042006",
            birthDate="1988-12-13",
        )

        account_result = asaas_svc.sub_accounts.create(request_subaccount)

        assert isinstance(account_result, SubAccountViewSchema)
        assert account_result.name == "Porco Rosso"
        assert isinstance(account_result.walletId, str)
        assert isinstance(account_result.apiKey, str)
        assert account_result.apiKey.startswith("$aact_YT")

    def test_get_subaccount_by_id(self, asaas_svc):
        account_result = asaas_svc.sub_accounts.get_by_id(
            "a7e73f10-eb87-42e6-b93b-9949917e8134"
        )
        assert isinstance(account_result, SubAccountViewSchema)
        assert account_result.name == "Porco Rosso"
        assert account_result.apiKey is None

    def test_get_subaccount_list(self, asaas_svc):
        sub_accounts = asaas_svc.sub_accounts.list()
        assert sub_accounts.data[0].name == "Porco Rosso"
        assert sub_accounts.data[0].apiKey is None
        assert isinstance(sub_accounts.data[0], SubAccountViewSchema)



from typing import Optional, Literal, List
from asaaspy.schemas.base import BaseSchema
from asaaspy.schemas.base_fields import DateTime, Date
from schemas.base import PaginatedQueryParams


class WebhookSchema(BaseSchema):
    apiVersion: Optional[int] = 3
    sendType: Literal["SEQUENTIALLY", "NON_SEQUENTIALLY"] = "SEQUENTIALLY"
    name: Optional[str] = None
    url: Optional[str] = None
    email: Optional[str] = None
    enabled: Optional[bool] = None
    interrupted: Optional[bool] = None
    authToken: Optional[str] = None
    events: Optional[List[str]] = None


class SubAccountSchema(BaseSchema):
    name: str
    email: str
    cpfCnpj: str
    mobilePhone: str
    incomeValue: int
    address: str
    addressNumber: str
    province: str  # Bairro
    postalCode: str
    webhooks: List[WebhookSchema] = []
    companyType: Optional[Literal["MEI", "LIMITED", "INDIVIDUAL", "ASSOCIATION"]] = None
    birthDate: Optional[Date] = None
    loginEmail: Optional[str] = None
    phone: Optional[str] = None
    site: Optional[str] = None
    complement: Optional[str] = None


class SubAccountBankAccountViewSchema(BaseSchema):
    agency: str
    account: str
    accountDigit: str


class SubAccountCommercialInfoExpirationViewSchema(BaseSchema):
    isExpired: Optional[bool]
    scheduledDate: Optional[DateTime]


class SubAccountViewSchema(BaseSchema):
    object: str
    id: str
    name: str
    email: str
    mobilePhone: str
    address: str
    addressNumber: str
    province: str
    postalCode: str
    cpfCnpj: str
    state: str
    walletId: str
    apiKey: Optional[str] = None
    accountNumber: SubAccountBankAccountViewSchema
    loginEmail: Optional[str] = None
    phone: Optional[str] = None
    complement: Optional[str] = None
    birthDate: Optional[str] = None
    personType: Optional[Literal["JURIDICA", "FISICA"]] = None
    companyType: Optional[Literal["MEI", "LIMITED", "INDIVIDUAL", "ASSOCIATION"]] = None
    city: Optional[int] = None
    country: Optional[str] = None
    tradingName: Optional[str] = None
    site: Optional[str] = None
    commercialInfoExpiration: Optional[SubAccountCommercialInfoExpirationViewSchema] = (
        None
    )


class SubAccountFilterBySchema(PaginatedQueryParams):
    cpfCnpj: Optional[str] = None
    email: Optional[str] = None
    name: Optional[str] = None
    walletId: Optional[str] = None



from asaaspy.client.base import AsaasResource
from schemas.v3.subaccount import (
    SubAccountViewSchema,
    SubAccountSchema,
    SubAccountFilterBySchema,
)


class SubAccountsResource(AsaasResource):
    def create(self, sub_account_schema: SubAccountSchema) -> SubAccountViewSchema:
        response = self.call(
            "POST", "v3/accounts", json=sub_account_schema.as_lean_dict()
        )
        return SubAccountViewSchema(**response)

    def get_by_id(self, id):
        response = self.call("GET", f"v3/accounts/{id}")
        return SubAccountViewSchema(**response)

    def list(self, **filters):
        filters = SubAccountFilterBySchema(**filters)
        return self.get_list_response(
            self.call("GET", "v3/accounts", params=filters.as_lean_dict()),
            SubAccountViewSchema,
        )
